@@ -1,5 +1,5 @@
 /* audispd-config.c -- 
- * Copyright 2007-08,2010 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2007-08,2010,2014 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -56,7 +56,8 @@ struct nv_list
 	int option;
 };
 
-static char *get_line(FILE *f, char *buf);
+static char *get_line(FILE *f, char *buf, unsigned size, int *lineno,
+		const char *file);
 static int nv_split(char *buf, struct nv_pair *nv);
 static const struct kw_pair *kw_lookup(const char *val);
 static int q_depth_parser(struct nv_pair *nv, int line, 
@@ -122,7 +123,7 @@ int load_config(daemon_conf_t *config, const char *file)
 	int fd, rc, mode, lineno = 1;
 	struct stat st;
 	FILE *f;
-	char buf[128];
+	char buf[160];
 
 	clear_config(config);
 
@@ -178,7 +179,7 @@ int load_config(daemon_conf_t *config, const char *file)
 		return 1;
 	}
 
-	while (get_line(f, buf)) {
+	while (get_line(f, buf, sizeof(buf), &lineno, file)) {
 		// convert line into name-value pair
 		const struct kw_pair *kw;
 		struct nv_pair nv;
@@ -208,6 +209,8 @@ int load_config(daemon_conf_t *config, const char *file)
 		}
 		if (nv.value == NULL) {
 			fclose(f);
+			audit_msg(LOG_ERR, 
+				"Not processing any more lines in %s", file);
 			return 1;
 		}
 
@@ -247,14 +250,31 @@ int load_config(daemon_conf_t *config, const char *file)
 	return 0;
 }
 
-static char *get_line(FILE *f, char *buf)
+static char *get_line(FILE *f, char *buf, unsigned size, int *lineno,
+	const char *file)
 {
-	if (fgets_unlocked(buf, 128, f)) {
+	int too_long = 0;
+
+	while (fgets_unlocked(buf, size, f)) {
 		/* remove newline */
 		char *ptr = strchr(buf, 0x0a);
-		if (ptr)
-			*ptr = 0;
-		return buf;
+		if (ptr) {
+			if (!too_long) {
+				*ptr = 0;
+				return buf;
+			}
+			// Reset and start with the next line
+			too_long = 0;
+			*lineno = *lineno + 1;
+		} else {
+			// If a line is too long skip it.
+			//  Only output 1 warning
+			if (!too_long)
+				audit_msg(LOG_ERR,
+					"Skipping line %d in %s: too long",
+					*lineno, file);
+			too_long = 1;
+		}
 	}
 	return NULL;
 }
