@@ -186,6 +186,7 @@ extern "C" {
 #define AUDIT_FS_RELABEL		2309 /* Filesystem relabeled */
 #define AUDIT_USER_MAC_POLICY_LOAD	2310 /* Userspc daemon loaded policy */
 #define AUDIT_ROLE_MODIFY		2311 /* Admin modified a role */
+#define AUDIT_USER_MAC_CONFIG_CHANGE	2312 /* Change made to MAC policy */
 
 #define AUDIT_FIRST_CRYPTO_MSG		2400
 #define AUDIT_CRYPTO_TEST_USER		2400 /* Crypto test results */
@@ -228,6 +229,15 @@ extern "C" {
 #define AUDIT_SECCOMP		1326 /* Secure Computing event */
 #endif
 
+#ifndef AUDIT_PROCTITLE
+#define AUDIT_PROCTITLE		1327 /* Process Title info */
+#endif
+
+#undef AUDIT_FEATURE_CHANGE
+#ifndef AUDIT_FEATURE_CHANGE
+#define AUDIT_FEATURE_CHANGE	1328 /* Audit feature changed value */
+#endif
+
 #ifndef AUDIT_ANOM_LINK
 #define AUDIT_ANOM_LINK		1702 /* Suspicious use of file links */
 #endif
@@ -416,7 +426,7 @@ typedef enum {
 	MACH_S390X,
 	MACH_S390,
 	MACH_ALPHA,
-	MACH_ARMEB,
+	MACH_ARM,
 	MACH_AARCH64
 } machine_t;
 
@@ -441,6 +451,7 @@ extern int  audit_get_reply(int fd, struct audit_reply *rep, reply_t block,
 extern uid_t audit_getloginuid(void);
 extern int  audit_setloginuid(uid_t uid);
 extern int  audit_detect_machine(void);
+extern int audit_determine_machine(const char *arch);
 
 /* Translation functions */
 extern int        audit_name_to_field(const char *field);
