@@ -1,5 +1,5 @@
 /* s390x_table.h --
- * Copyright 2005-06,2008-13 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-06,2008-14 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -306,3 +306,5 @@ _S(341, "process_vm_writev")
 _S(342, "s390_runtime_instr")
 _S(343, "kcmp")
 _S(344, "finit_module")
+_S(345, "sched_setattr")
+_S(346, "sched_getattr")
