@@ -1,5 +1,5 @@
 /* auditd-config.c -- 
- * Copyright 2004-2011,2013 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2011,2013-14 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -62,7 +62,8 @@ struct nv_list
 	int option;
 };
 
-static char *get_line(FILE *f, char *buf);
+static char *get_line(FILE *f, char *buf, unsigned size, int *lineno,
+		const char *file);
 static int nv_split(char *buf, struct nv_pair *nv);
 static const struct kw_pair *kw_lookup(const char *val);
 static int log_file_parser(struct nv_pair *nv, int line, 
@@ -234,7 +235,7 @@ void set_allow_links(int allow)
 /*
  * Set everything to its default value
 */
-static void clear_config(struct daemon_conf *config)
+void clear_config(struct daemon_conf *config)
 {
 	config->qos = QOS_NON_BLOCKING;
 	config->sender_uid = 0;
@@ -281,7 +282,7 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 	int fd, rc, mode, lineno = 1;
 	struct stat st;
 	FILE *f;
-	char buf[128];
+	char buf[160];
 
 	clear_config(config);
 	log_test = lt;
@@ -342,7 +343,7 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 		return 1;
 	}
 
-	while (get_line(f, buf)) {
+	while (get_line(f, buf, sizeof(buf), &lineno, CONFIG_FILE)) {
 		// convert line into name-value pair
 		const struct kw_pair *kw;
 		struct nv_pair nv;
@@ -372,6 +373,9 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 		}
 		if (nv.value == NULL) {
 			fclose(f);
+			audit_msg(LOG_ERR,
+				"Not processing any more lines in %s",
+				CONFIG_FILE);
 			return 1;
 		}
 
@@ -411,14 +415,31 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 	return 0;
 }
 
-static char *get_line(FILE *f, char *buf)
+static char *get_line(FILE *f, char *buf, unsigned size, int *lineno,
+	const char *file)
 {
-	if (fgets_unlocked(buf, 128, f)) {
+	int too_long = 0;
+
+	while (fgets_unlocked(buf, size, f)) {
 		/* remove newline */
 		char *ptr = strchr(buf, 0x0a);
-		if (ptr)
-			*ptr = 0;
-		return buf;
+		if (ptr) {
+			if (!too_long) {
+				*ptr = 0;
+				return buf;
+			}
+			// Reset and start with the next line
+			too_long = 0;
+			*lineno = *lineno + 1;
+		} else {
+			// If a line is too long skip it.
+			// Only output 1 warning
+			if (!too_long)
+				audit_msg(LOG_ERR,
+					"Skipping line %d in %s: too long",
+					*lineno, file);
+			too_long = 1;
+		}
 	}
 	return NULL;
 }
